/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.jangle;

import java.util.*;
import java.io.File;
import net.percederberg.mibble.*;
import net.percederberg.mibble.value.*;

/**
 * SnmpMibManager provides a mechanism to load and query MIB files.
 */
public class SnmpMibManager {

    private Map <String, String> oid2nameMap;
    private Map <String, String> name2oidMap;
    private MibLoader mibloader;

    public SnmpMibManager() {
	oid2nameMap = new HashMap <String, String> ();
	name2oidMap = new HashMap <String, String> ();
	mibloader = new MibLoader();
	// solaris/opensolaris
	readMIBs("/etc/sma/snmp/mibs");
	// ubuntu
	readMIBs("/usr/share/snmp/mibs");
    }

    private void readMIBs(String location) {
	File f1 = new File(location);
	if (f1.exists()) {
	    mibloader.addDir(f1);
	    List <File> flist = Arrays.asList(f1.listFiles());
	    for (File f : flist) {
		try {
		    mibloader.load(f);
		} catch (Exception e) {}
	    }
	    Mib[] mibs = mibloader.getAllMibs();
	    List <Mib> miblist = Arrays.asList(mibs);
	    for (Mib mib : miblist) {
		scanMib(mib);
	    }
	}
    }

    private void scanMib(Mib mib) {
	for (Object o : mib.getAllSymbols()) {
	    MibSymbol  symbol = (MibSymbol) o;
	    MibValue value = extractOid(symbol);
	    if (value != null) {
		name2oidMap.put(symbol.toString(), value.toString());
		oid2nameMap.put(value.toString(), symbol.toString());
	    }
	}
    }

    public ObjectIdentifierValue extractOid(MibSymbol symbol) {
	MibValue  value;
	if (symbol instanceof MibValueSymbol) {
	    value = ((MibValueSymbol) symbol).getValue();
	    if (value instanceof ObjectIdentifierValue) {
		return (ObjectIdentifierValue) value;
	    }
	}
	return null;
    }

    /**
     * Return the MIB entry for this exact OID string.
     */
    public String getNameForOID(String oid) {
	return oid2nameMap.get(oid);
    }

    private String getParentOID(String s) {
	int i = s.lastIndexOf('.');
	if (i < 0) {
	    return null;
	}
	return s.substring(0, i);
    }

    /**
     * Return the closest MIB entry for this OID string.
     */
    public String getNearestNameForOID(String oid) {
	String namepar = oid2nameMap.get(oid);
	if (namepar != null) {
	    return namepar;
	}
	String spar = getParentOID(oid);
	while (spar != null) {
	    namepar = oid2nameMap.get(spar);
	    if (namepar != null) {
		return namepar;
	    }
	    spar = getParentOID(spar);
	}
	return null;
    }

    public String getOIDForName(String name) {
	return name2oidMap.get(name);
    }
}
